# RSQL 
# Introducing R

# This is a comment

# Basic functions
# R Contributors
contributors();
# Help about options
help(options);
# List of the current objects in the workspace
objects();

# Basic expressions
1 + 1;
2 + 3 * 4;
3 ^ 3;
sqrt(81);
pi;

# Variables
x <- 2;
y <- 3;
z <- 4;
x + y * z;
# Names are case-sensitive
X + Y + Z;
# Can use period
This.Year <- 2017;
This.Year;
# Equals as an assigment operator
x = 2;
y = 3;
z = 4;
x + y * z;
# Boolean equality test
x = 2;    # assignment
x;
x == 2;   # TRUE

# Vectors
rep(1,10);                 # replicate a number n times
seq(3,7);                  # Sequence of integers between 3 and 7
3:7;                       # Equivalent sequence
seq(5,17,by=3);            # Every 3rd integer between 5 and 17
# Vector variables
x <- c(2,0,0,4);           # A vector with elements 2,0,0,4
assign("y", c(1,9,9,9));   # Another way to assign a vector
c(5,4,3,2) -> z;           # Another way to assign a vector      
q = c(1,2,3,4);            # Another way to assign a vector
# Vector operations
x + y                     # Sums elements of two vectors
x * 4                     # Multiplies elements
sqrt(x)                   # Function applies to each element
# Vector elements
x <- c(2,0,0,4);
x[1];               # Select the first element
x[-1];              # Exclude the first element
x[1] <- 3; x;       # Assign a value to the first element
x[-1] = 5; x;       # Assign a value to all other elements
y <- c(1,9,9,9);
y < 8;              # Compares each element, returns result as vector
y[4] = 1;
y < 8;
y[y<8] = 2; y;      # Edits elements marked as TRUE in index vector

# Matrices, Arrays, factors, lists, data frames
# Array
x = c(1,2,3,4,5,6); x;         # A simple vector
Y = array(x, dim=c(2,3)); Y;   # A matrix from the vector - fill by columns
Z = matrix(x,2,3,byrow=F); Z;  # A matrix from the vector - fill by columns
U = matrix(x,2,3,byrow=T); U;  # A matrix from the vector - fill by rows

# Matrix operations
U <- matrix(c(1,2,3,4), 2, 2, byrow=T);
V <- matrix(c(5,6,7,8), 2, 2, byrow=T);
A <- matrix(c(1,2,3,4,5,6), 2, 3, byrow=T);
B <- matrix(c(1,2,3,4,5,6), 3, 2, byrow=T);

U; V;
A; B;

U + V; V + U;        # Addition
V - U; U - V;        # Subtraction
U %*% V; V %*% U;    # Multiplication

U %*% A;             # Conformable
U %*% B;             # Non-comformable

cbind(U, V);         # Combine by columns
rbind(V, U);         # Combine by rows

cbind(U, A);         # Combine by columns
cbind(U, B);         # Combine by columns - error, number of rows must match
rbind(U, A);         # Combine by rows - error, number of columns must match
rbind(U, B);         # Combine by rows


# Factor
x = c("good", "moderate", "good", "bad", "bad", "good");
y = factor(x); y;  
z = factor(x, order=TRUE); z;
w = factor(x, order=TRUE, 
           levels=c("bad", "moderate","good")); w;

# List
L = list(name1="ABC", name2="DEF",
         no.children=2, children.ages=c(3,6));
L;
L[[1]];
L[[4]];
L[[4]][2];

# Data frame
# Note that you need to escape the backslash character
PC = read.table("C:\\IntroducingR\\ProductCategories.txt",
                sep=",", header=TRUE);
PC;


# Reading from SQL Server

# Install RODBC library
install.packages("RODBC");

# Load RODBC library
library(RODBC);

# Connect to AWDW
# AWDW system DSN created in advance
con <- odbcConnect("AWDW", uid="RUser", pwd="Pa$$w0rd");

# Read data from a view
TM <- as.data.frame(sqlQuery(con, 
 "SELECT CustomerKey, MaritalStatus, Gender,
   TotalChildren, NumberChildrenAtHome,
   EnglishEducation AS Education,
   EnglishOccupation AS Occupation,
   HouseOwnerFlag, NumberCarsOwned, CommuteDistance,
   Region, BikeBuyer,
   Age, YearlyIncome
  FROM dbo.vTargetMail"),
 stringsAsFactors = TRUE);

TM;

close(con);

# Accessing data in a data frame
TM[2:3];                              # Two columns
TM[c("MaritalStatus", "Gender")];     # Two columns
TM[1:3,2:3];                          # Three rows, two columns
TM[1:3,c("MaritalStatus", "Gender")]; # Three rows, two columns

# $ Notation
table(TM$MaritalStatus, TM$Gender);
attach(TM);
table(MaritalStatus, Gender);
detach(TM);
with(TM,
     {table(MaritalStatus, Gender)});


# Value labels
table(TM$BikeBuyer, TM$Gender);
TM$BikeBuyer <- factor(TM$BikeBuyer,
                       levels = c(0,1),
                       labels = c("No","Yes"));
table(TM$BikeBuyer, TM$Gender);

# Basic graphics
attach(TM);

# A simple distribution
plot(Education);

# Education is ordered
Education = factor(Education, order=TRUE, 
                   levels=c("Partial High School", 
                            "High School","Partial College",
                            "Bachelors", "Graduate Degree"));
plot(Education, main = 'Education',
     xlab='Education', ylab ='Number of Cases',
     col="purple");


# Generating a subset data frame
cols1 <- c("CustomerKey", "NumberCarsOwned", "TotalChildren");
TM1 <- TM[TM$CustomerKey < 11010, cols1];
names(TM1) <- c("CustomerKey1", "NumberCarsOwned1", "TotalChildren1");
attach(TM1);

# Generating a table from NumberCarsOwned and BikeBuyer
nofcases <- table(NumberCarsOwned, BikeBuyer);
nofcases;

# Saving parameters
oldpar <- par(no.readonly = TRUE);

# Defining a 2x2 graph
par(mfrow=c(2,2));

# Education and marital status
plot(Education, MaritalStatus,
     main='Education and marital status',
     xlab='Education', ylab ='Marital Status',
     col=c("blue", "yellow"));

# Histogram with a title and axis labels and color
hist(NumberCarsOwned, main = 'Number of cars owned',
     xlab='Number of Cars Owned', ylab ='Number of Cases',
     col="blue");

# Plot with two lines, title, legend, and axis legends
plot_colors=c("blue", "red");
plot(TotalChildren1, 
     type="o",col='blue', lwd=2,
     xlab="Key",ylab="Number");
lines(NumberCarsOwned1, 
      type="o",col='red', lwd=2);
legend("topleft", 
       c("TotalChildren", "NumberCarsOwned"),
       cex=1.4,col=plot_colors,lty=1:2,lwd=1, bty="n");
title(main="Total children and number of cars owned line chart", 
      col.main="DarkGreen", font.main=4);

# NumberCarsOwned and BikeBuyer grouped bars
barplot(nofcases,
        main='Number of cars owned and bike buyer gruped',    
        xlab='BikeBuyer', ylab ='NumberCarsOwned',
        legend=rownames(nofcases),
        col=c("black", "blue", "red", "orange", "yellow"),
        beside=TRUE);

# Restoring the default graphical parameters
par(oldpar);

# removing the data frames from the search path
detach(TM);
detach(TM1);


# Basic statistics
attach(TM);

# A quick summary for the whole dataset
summary(TM);

# A quick summary for Age
summary(Age);
# Details for Age
mean(Age);
median(Age);
min(Age);
max(Age);
range(Age);
quantile(Age, 1/4);
quantile(Age, 3/4);
IQR(Age);
var(Age);
sd(Age);

# Custom function for skewness and kurtosis
skewkurt <- function(p){
  avg <- mean(p)
  cnt <- length(p)
  stdev <- sd(p)
  skew <- sum((p-avg)^3/stdev^3)/cnt
  kurt <- sum((p-avg)^4/stdev^4)/cnt-3
  return(c(skewness=skew, kurtosis=kurt))
};
skewkurt(Age);


# ggplot2
# Install and load the ggplot library
install.packages("ggplot2", dependencies = TRUE);
library(ggplot2);

# Plot with count (number) Education by Region
ggplot (TM, aes(Region, fill=Education)) + 
  geom_bar();

ggplot (TM, aes(Region, fill=Education)) +
  geom_bar(position="fill");

ggplot (TM, aes(Region, fill=Education)) +
  geom_bar(position="dodge");

# Trellis chart - 
# plots two separate bar plots "MaritalStatus" one for Married and other for Single 
# with count by "NumberCarsOwned" 
ggplot(TM, aes(NumberCarsOwned) ) + 
  geom_histogram(color = "white") + 
  facet_grid(MaritalStatus ~ .);
# Another trellis chart
ggplot(data=TM,aes(x=Education,y=BikeBuyer,fill=Region))+
  geom_bar(stat="identity")+
  facet_grid(.~Region)+  # Trellis command
  theme(legend.position="none",axis.text.x=element_text(angle=45));


# Data mining

# Decision Trees 
# Package party (Decision Trees)
install.packages("party", dependencies = TRUE);
library("party");

# Train the model
TMDT <- ctree(BikeBuyer ~ NumberCarsOwned + Region,
              data = TM);

# Show the results
plot(TMDT, type = "simple");


# Naive Bayes
# Package e1071 (Naive Bayes)
install.packages("e1071", dependencies = TRUE);
library(e1071);

# Build the Naive Bayes model
TMNB <- naiveBayes(TM[,2:11], TM[,12]);

# Apriori probabilities for the target variable (BikeBuyer)
TMNB$apriori;
# Apriori probabilities for the input variables in classes of the target variable
TMNB$tables;

# Predicitions
predict(TMNB, TM, type = "raw");

# Data frame with predictions for all rows
TM_PR <- as.data.frame(predict(
  TMNB, TM, type = "raw"));

# Combine original data with predictions
df_TM_PR <- cbind(TM, TM_PR);
# View the original data with the predictions
View(df_TM_PR);

# End of Introducing R
