-- R Services post installation tasks

USE master;
EXEC sp_configure 'show advanced options', 1;
RECONFIGURE
EXEC sp_configure 'external scripts enabled', 1; 
RECONFIGURE;
GO

-- Check
EXEC sp_configure;
GO

-- Check installed packages
EXECUTE sp_execute_external_script
 @language=N'R'
,@script = 
 N'str(OutputDataSet);
   packagematrix <- installed.packages();
   NameOnly <- packagematrix[,1];
   OutputDataSet <- as.data.frame(NameOnly);'
,@input_data_1 = N'SELECT 1 AS col'
WITH RESULT SETS (( PackageName nvarchar(250) ));
GO

-- Add Ruser
USE master;
GO
CREATE LOGIN RUser WITH PASSWORD=N'Pa$$w0rd';
GO
USE AdventureworksDW2014;
GO
CREATE USER RUser FOR LOGIN RUser;
GO
ALTER ROLE db_datareader ADD MEMBER RUser;
GO

USE AdventureworksDW2014;
GO

-- Read R demo data Iris in SQL Server
EXECUTE sp_execute_external_script
  @language = N'R'
 ,@script = N'iris_data <- iris;'
 ,@input_data_1 = N''
 ,@output_data_1_name = N'iris_data'
WITH RESULT SETS 
(
 ("Sepal.Length" FLOAT NOT NULL, 
  "Sepal.Width"  FLOAT NOT NULL,
  "Petal.Length" FLOAT NOT NULL, 
  "Petal.Width"  FLOAT NOT NULL, 
  "Species"      VARCHAR(100) NOT NULL)
);
GO

-- Install package e1071
-- Run as administator R.exe R command prompt from the
-- C:\Program Files\Microsoft SQL Server\MSSQL13.MSSQLSERVER\R_SERVICES\bin
-- folder and execute the following command:
-- install.packages("e1071");
-- Also, for the PowerBI Desktop, install:
-- install.packages("RODBC");
-- install.packages("ggplot2");
-- q();

-- Use SQL Server data and analyze it in R
EXECUTE sp_execute_external_script
  @language = N'R'
 ,@script = N'
   library(e1071);
   TMNB <- naiveBayes(TM[,2:11], TM[,12]);
   TM_PR <- as.data.frame(predict(TMNB, TM, type = "raw"));
   # Combine original data with predictions
   df_TM_PR <- cbind(TM[,-(2:11)], TM_PR);'
 ,@input_data_1 = N'
     SELECT CustomerKey, MaritalStatus, Gender,
      TotalChildren, NumberChildrenAtHome,
      EnglishEducation AS Education,
      EnglishOccupation AS Occupation,
      HouseOwnerFlag, NumberCarsOwned, CommuteDistance,
      Region, BikeBuyer
     FROM dbo.vTargetMail;'
 ,@input_data_1_name =  N'TM'
 ,@output_data_1_name = N'df_TM_PR'
WITH RESULT SETS 
(
 ("CustomerKey"             INT   NOT NULL,
  "BikeBuyer"               INT   NOT NULL,
  "Predicted_0_Probability" FLOAT NOT NULL, 
  "Predicted_1_Probability" FLOAT NOT NULL)
);
GO


-- Using with SSRS

-- Create table to store the plot
CREATE TABLE dbo.R_models
(
 Rmodel VARBINARY(MAX) NOT NULL
);
GO

-- Wrapper procedure
CREATE PROCEDURE dbo.RPlot AS
EXECUTE sp_execute_external_script
  @language = N'R'
 ,@script = N'
   image_file = tempfile();
   jpeg(filename = image_file);
   with(df_TM, {
   hist(NumberCarsOwned, main = "Number of Cars Owned",
        xlab="Number of Cars Owned", ylab ="Number of Cases",
        col="purple")
   });
   dev.off();
   df_XX <- data.frame(data=readBin(file(image_file, "rb"), what=raw(), n=1e6));
  '
 ,@input_data_1 = N'
     SELECT CustomerKey, 
	  TotalChildren, 
      NumberCarsOwned
     FROM dbo.vTargetMail;'
 ,@input_data_1_name =  N'df_TM'
 ,@output_data_1_name = N'df_XX'
WITH RESULT SETS 
(
 (Plot VARBINARY(max))
);
GO
-- Test
EXEC dbo.RPlot;
GO

-- Insert into a table
INSERT INTO dbo.R_models (Rmodel)
EXEC dbo.RPlot;
GO
-- Query
SELECT TOP 1 Rmodel FROM dbo.R_models;
GO


-- Clean up, if needed
TRUNCATE TABLE dbo.R_models;
GO
DROP PROCEDURE dbo.RPlot;
DROP TABLE dbo.R_models;
GO
